//=====================================================================
//	    :
//=====================================================================

//---------------------------------------------------------------------
#property copyright 	"Dima S., 2010 ."
#property link      	"dimascub@mail.com"
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	 :
//---------------------------------------------------------------------
//	08.11.2010. - V1.00
//	 -  ;
//
//	16.11.2010. - V1.01
//	 -  -  ;
//
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	 :
//---------------------------------------------------------------------

#include	<Object.mqh>
#include	<Arrays\List.mqh>
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	 :
//---------------------------------------------------------------------
class TExtremum : public CObject
{
private:
	datetime			extr_datetime;																				// /   
	double				extr_price;																						//    
	
protected:
	virtual int		Compare(const CObject* _node, int _mode = 0) const;

public:
	void					TExtremum();																					// 
	void					~TExtremum();																					// 
	void					SetExtremum(datetime _time, double _price);						//  /     
	void					SetDateTime(datetime _time);													//  /   
	void					SetPrice(double _price);															//     

public:
	datetime			GetDateTime() const;																	//  /   
	double				GetPrice() const;																			//     

public:
  virtual bool	SaveExtremum(string _dt_name, string _p_name);				//  
  virtual bool	LoadExtremum(string _dt_name, string _p_name);				//  
  virtual bool	DeleteExtremum(string _dt_name, string _p_name);			//  
};
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void	TExtremum::TExtremum()
{
	this.extr_datetime = 0;
	this.extr_price = 0.0;
}

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void	TExtremum::~TExtremum()
{
}

//---------------------------------------------------------------------
//	  (/):
//---------------------------------------------------------------------
bool	TExtremum::SaveExtremum(string _dt_name, string _p_name)
{
	datetime  dt_result = GlobalVariableSet(_dt_name, (double)this.extr_datetime);
	datetime  p_result = GlobalVariableSet(_p_name, (double)this.extr_price);
	if(dt_result != 0 && p_result != 0)
	{
		return(true);
	}

	return(false);
}

//---------------------------------------------------------------------
//	  (/):
//---------------------------------------------------------------------
bool	TExtremum::LoadExtremum(string _dt_name, string _p_name)
{
	double	dt_temp, p_temp;
	bool		result = GlobalVariableGet(_dt_name, dt_temp);
	result &= GlobalVariableGet(_p_name, p_temp);
	if(result != false)
	{
		this.extr_datetime = (datetime)dt_temp;
		this.extr_price = p_temp;
		return(true);
	}

	return(false);
}

//---------------------------------------------------------------------
//	 :
//---------------------------------------------------------------------
bool	TExtremum::DeleteExtremum(string _dt_name, string _p_name)
{
	bool		result = GlobalVariableDel(_dt_name);
	result &= GlobalVariableDel(_p_name);
	return(result);
}

//---------------------------------------------------------------------
//	    :
//---------------------------------------------------------------------
int	TExtremum::Compare(const CObject* _node, int _mode = 0) const
{
	datetime		temp = ((TExtremum*)_node).GetDateTime();
	datetime		curr = this.GetDateTime();
	if(curr > temp)
	{
		return(_mode > 0 ? 1 : -1);
	}
	else if(curr < temp)
	{
		return(_mode > 0 ? -1 : 1);
	}

	return(0);
}

//---------------------------------------------------------------------
//	 /     :
//---------------------------------------------------------------------
void	TExtremum::SetExtremum(datetime _time, double _price)
{
	this.extr_datetime = _time;
	this.extr_price = _price;
}

//---------------------------------------------------------------------
//	 /   :
//---------------------------------------------------------------------
void	TExtremum::SetDateTime(datetime _time)
{
	this.extr_datetime = _time;
}

//---------------------------------------------------------------------
//	    :
//---------------------------------------------------------------------
void	TExtremum::SetPrice(double _price)
{
	this.extr_price = _price;
}

//---------------------------------------------------------------------
//	 /   :
//---------------------------------------------------------------------
datetime	TExtremum::GetDateTime() const
{
	return(this.extr_datetime);
}

//---------------------------------------------------------------------
//	    :
//---------------------------------------------------------------------
double	TExtremum::GetPrice() const
{
	return(this.extr_price);
}

//---------------------------------------------------------------------
//	 :
//---------------------------------------------------------------------
class TExtremumList : public CList
{
private:
	string							channel_prefix;																	//   ()
	ENUM_TIMEFRAMES			chart_timeframe;																// - 
	string							chart_symbol;																		//    

protected:
	string	MakeDTimeName(int _nmb);																		//    / / 
	string	MakePriceName(int _nmb);																		//    /  

public:
	void		TExtremumList();																						// 
	void		~TExtremumList();																						// 
	void		SetChannalParams(string _pref, string _symbol = NULL, ENUM_TIMEFRAMES _curr_tf = PERIOD_CURRENT);
	void		AddExtremum(datetime _time, double _price);
	void		DeleteAllExtremum();
	void		SaveExtremumList();
	void		LoadExtremumList();
	int			FindExtremum(datetime _dt);																	//     

public:
	datetime	GetDateTime(int _index);
	double		GetPrice(int _index);
};
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void	TExtremumList::TExtremumList()
{
	this.chart_timeframe = PERIOD_CURRENT;
	this.channel_prefix = "";
	this.chart_symbol = NULL;
}

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
void	TExtremumList::~TExtremumList()
{
	this.Clear();
}

//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
void	TExtremumList::SetChannalParams(string _pref, string _symbol, ENUM_TIMEFRAMES _curr_tf)
{
	this.channel_prefix = _pref;

	if(_symbol == NULL)
	{
		this.chart_symbol = ChartSymbol();
	}
	else
	{
		this.chart_symbol = _symbol;
	}

	if(_curr_tf == 0)
	{
		this.chart_timeframe = ChartPeriod();
	}
	else
	{
		this.chart_timeframe = _curr_tf;
	}
}

//---------------------------------------------------------------------
//	    :
//---------------------------------------------------------------------
void	TExtremumList::DeleteAllExtremum()
{
	this.Clear();
}

//---------------------------------------------------------------------
//	       :
//---------------------------------------------------------------------
void	TExtremumList::AddExtremum(datetime _time, double _price)
{
//	 :
	TExtremum*	extr = new TExtremum();
	extr.SetExtremum(_time, _price);

//	   :
	this.Add(extr);

//	:
	this.Sort(1);
}

//---------------------------------------------------------------------
//	  :
//---------------------------------------------------------------------
void	TExtremumList::SaveExtremumList()
{
	int		k = 1;
	TExtremum*	extr = (TExtremum*)(this.GetFirstNode());
	while(extr != NULL)
	{
		extr.SaveExtremum(MakeDTimeName(k), MakePriceName(k));
		k++;
		extr = (TExtremum*)(this.GetNextNode());
	}
}

//---------------------------------------------------------------------
//	    :
//---------------------------------------------------------------------
int		TExtremumList::FindExtremum(datetime _dt)
{
	int		k = 0;
	TExtremum*	extr = (TExtremum*)(this.GetFirstNode());
	while(extr != NULL)
	{
		if(extr.GetDateTime() == _dt)
		{
			return(k);
		}
		extr = (TExtremum*)(this.GetNextNode());
	}
	return(-1);																																//   
}

//---------------------------------------------------------------------
//	 :
//---------------------------------------------------------------------
void	TExtremumList::LoadExtremumList()
{
//	    :
	this.DeleteAllExtremum();

	int		count = GlobalVariablesTotal();
	if(count <= 0)
	{
		return;
	}

	for(int i = (count - 1); i >= 0; i--)
	{
		string	name = GlobalVariableName(i);
		string	dt_name = MakeDTimeName(-1);
		int			index = StringFind(name, dt_name);
		if(index == -1 || index != 0)
		{
			continue;
		}

		string	p_name = MakePriceName(-1) + StringSubstr(name, StringLen(dt_name));
//	 :
		TExtremum*	extr = new TExtremum();
		extr.LoadExtremum(name, p_name);
		
		this.Add(extr);
	}

//	:
	this.Sort(1);
}

//---------------------------------------------------------------------
//	  / :
//---------------------------------------------------------------------
datetime	TExtremumList::GetDateTime(int _index)
{
	TExtremum*	extr = (TExtremum*)(this.GetNodeAtIndex(_index));
	if(extr != NULL)
	{
		return(extr.GetDateTime());
	}

	return(0);	
}

//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double	TExtremumList::GetPrice(int _index)
{
	TExtremum*	extr = (TExtremum*)(this.GetNodeAtIndex(_index));
	if(extr != NULL)
	{
		return(extr.GetPrice());
	}

	return(0);	
}

//---------------------------------------------------------------------
//	   / / :
//---------------------------------------------------------------------
string	TExtremumList::MakeDTimeName(int _nmb)
{
	string	name;
	if(_nmb > 0)
	{
		StringConcatenate(name, this.channel_prefix, "_", this.chart_symbol, "_", this.chart_timeframe, "_DTime_Extr", _nmb);
	}
	else
	{
		StringConcatenate(name, this.channel_prefix, "_", this.chart_symbol, "_", this.chart_timeframe, "_DTime_Extr");
	}
	return(name);
}

//---------------------------------------------------------------------
//	   /  :
//---------------------------------------------------------------------
string	TExtremumList::MakePriceName(int _nmb)
{
	string	name;
	if(_nmb > 0)
	{
		StringConcatenate(name, this.channel_prefix, "_", this.chart_symbol, "_", this.chart_timeframe, "_Price_Extr", _nmb);
	}
	else
	{
		StringConcatenate(name, this.channel_prefix, "_", this.chart_symbol, "_", this.chart_timeframe, "_Price_Extr");
	}
	return(name);
}
//---------------------------------------------------------------------
